/* $Id: FileInstanciator.java 319 2009-12-21 09:36:28Z yuki $ */
package saccubus.converter.filegetter;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import org.apache.commons.io.FilenameUtils;
import saccubus.ConvertStopFlag;
import saccubus.converter.profile.InputFileSetting;
import saccubus.net.TextProgressListener;

/**
 * _E[hSKvƂȂꍇ̃t@CCX^XNX.
 * @author yuki
 */
public class FileInstanciator {

    private final InstanciationType videoType;
    private final InstanciationType commentType;
    private final InstanciationType tcommType;
    private final String videoId;
    private FileGetter videoFileGetter;
    private FileGetter commentFileGetter;
    private FileGetter tcommFileGetter;

    protected void setVideoFileGetter(FileGetter getter) {
        this.videoFileGetter = getter;
    }

    protected void setCommentFileGetter(FileGetter getter) {
        this.commentFileGetter = getter;
    }

    protected void setTcommFileGetter(FileGetter getter) {
        this.tcommFileGetter = getter;
    }

    public static FileInstanciator create(
            ConvertStopFlag stopFlag,
            InstanciationType videoType,
            CommentInstanciationType commentType,
            InstanciationType tcommType,
            LoginInfo li,
            String tag, String time) throws
            IOException {
        FileInstanciator getter;
        if (videoType.isDoanload() || commentType.isDoanload() || tcommType.isDoanload()) {
            getter = new WebFileInstanciator(stopFlag, videoType, commentType, tcommType, li, tag, time);
        } else {
            getter = new FileInstanciator(videoType, commentType, tcommType, tag);
        }
        return getter;
    }

    protected FileInstanciator(
            InstanciationType videoType,
            InstanciationType commentType,
            InstanciationType tcommType,
            String videoId) {
        this.videoType = videoType;
        this.commentType = commentType;
        this.tcommType = tcommType;
        this.videoId = videoId;
        FileGetter getter = new FileGetter();
        setVideoFileGetter(getter);
        setCommentFileGetter(getter);
        setTcommFileGetter(getter);
    }

    /**
     * @return ̃^Cg.
     * @throws FileNotFoundException rfI^Cg̎AQlƂȂt@CȂB
     */
    public String getVideoTitle() throws FileNotFoundException {
        String fileName = null;
        if (!videoType.isAutoFileName()) {
            // t@C𒼐ڎw肵Ăꍇ́Ãt@CɃ^Cg擾.
            fileName = videoType.getInitFile().toString();
        } else {
            // ̏ꍇ́AfBNgɂt@C^CgސB
            String[] files = videoType.getInitFile().list();
            if (files != null) {
                for (String file : files) {
                    if (file.startsWith(getVideoIdWithBracket())) {
                        fileName = FilenameUtils.getBaseName(file);
                        break;
                    }
                }
            }
            if (fileName == null) {
                throw new FileNotFoundException(getVideoIdWithBracket() + "̃t@Cł܂łB");
            }
        }

        String baseName = FilenameUtils.getBaseName(fileName);
        int s = baseName.indexOf(getVideoIdWithBracket());
        return baseName.replace(getVideoIdWithBracket(), "");
    }

    /**
     * t@C擾܂.
     * @param listener iʒm󂯎邽߂̃Xi.
     * @return t@C.
     * @throws IOException t@C݂Ȃ, 擾Ɏs.
     */
    public final File getVideoFile(TextProgressListener listener) throws IOException {
        File file = new FileLocator(videoType.isAutoFileName(), videoType.getInitFile(), getVideoIdWithBracket(),
                getVideoTitle(), ".flv").getFile();
        file = videoFileGetter.get(file, listener);
        return file;
    }

    /**
     * Rgt@C擾܂.
     * @param listener iʒm󂯎邽߂̃Xi.
     * @return Rgt@C.
     * @throws IOException Rgt@C݂Ȃ, 擾Ɏs.
     */
    public final File getCommentFile(TextProgressListener listener) throws IOException {
        File file = new FileLocator(commentType.isAutoFileName(), commentType.getInitFile(), getVideoIdWithBracket(),
                getVideoTitle(), ".xml").getFile();
        file = commentFileGetter.get(file, listener);
        return file;
    }

    /**
     * e҃Rgt@C擾܂.
     * @param listener iʒm󂯎邽߂̃Xi.
     * @return e҃Rgt@C.
     * @throws IOException Rgt@C݂Ȃ, 擾Ɏs.
     */
    public final File getTcommFile(TextProgressListener listener) throws IOException {
        File file = new FileLocator(tcommType.isAutoFileName(), tcommType.getInitFile(), getVideoIdWithBracket(),
                getVideoTitle(), ".xml").getFile();
        file = tcommFileGetter.get(file, listener);
        return file;
    }

    private String getVideoIdWithBracket() {
        return "[" + videoId + "]";
    }

    public static class InstanciationType {

        private final boolean download;
        private final boolean autoFileName;
        private final File initFile;

        /**
         * t@CCX^X@w肷NX.
         */
        public InstanciationType(InputFileSetting fileSetting) {
            this.download = fileSetting.isDownload();
            this.autoFileName = !fileSetting.getFile().isFile();
            this.initFile = fileSetting.getFile().getFile();
        }

        public boolean isDoanload() {
            return download;
        }

        public File getInitFile() {
            return initFile;
        }

        public boolean isAutoFileName() {
            return autoFileName;
        }
    }

    public static class CommentInstanciationType extends InstanciationType {

        private final boolean autoCommentNum;
        private final int backComment;

        public CommentInstanciationType(InputFileSetting fileSetting, boolean autoCommentNum, int backComment) {
            super(fileSetting);
            this.autoCommentNum = autoCommentNum;
            this.backComment = backComment;
        }

        public boolean isAutoCommentNum() {
            return autoCommentNum;
        }

        public int getBackComment() {
            return backComment;
        }
    }
}
